require 'uri'
require 'helix_web_services_client/open_model'

class HelixWebServicesClient

  #
  # All actions available to user when it comes to managing git fusion repositories
  #

  # Return list of configured repositories. It will fetch all configured repositories returning
  # a list of hashes with id and name of the repository, where id is a git-fusion encoded name,
  # and name is a full human-readable string.
  #
  def list_repos
    execute_method_no_body(:get, '/git-fusion/v1/repos')
  end

  # Return the repository details. For specified repository it will fetch and return
  # currently submitted configuration.
  #
  # @param repo_name [String] Repository name to be encoded to git-fusion standards
  def repo(repo_name)
    path = "/git-fusion/v1/repos/#{GitFusionStrings.encode(repo_name)}"
    execute_method_no_body(:get, path)
  end

  # # Create a new repository in GitFusion. This will upload a specified config
  # # to notify GF about creation of a new repository for git and perforce use.
  # #
  # # @param config [String] GitFusion config file
  # def new_repo(config)
  #   execute_method_with_body(:post, '/git-fusion/v1/repos', {'config' => config})
  # end

  # Replace config for a specified repository. This will replace current config
  # for a Git Fusion repo with a new one under the same name, for the same path.
  #
  # @param repo_name [String] Repository name to be encoded to git-fusion standards
  # @param config [String] GitFusion config file
  def submit_config(repo_name, config)
    path = "/git-fusion/v1/repos/#{GitFusionStrings.encode(repo_name)}"
    execute_method_with_body(:put, path, {'config' => config})
  end

  # Modify config for a specified repository. This will update chosen parts of p4gf_file.
  #
  # @param repo_name [String] Repository name to be encoded to git-fusion standards
  # @param config [String] String to update in current p4gf_config file
  def update_config(repo_name, config)
    execute_method_with_body(:patch, "/git-fusion/v1/repos/#{GitFusionStrings.encode(repo_name)}", {'config' => config})
  end

  # Delete a repository. This will remove config file for specified repository,
  # removing it in result.
  #
  # @param repo_name [String] Repository name to be encoded to git_fusion standards
  def delete_repo(repo_name)
    path = "/git-fusion/v1/repos/#{GitFusionStrings.encode(repo_name)}"
    execute_method_no_body(:delete, path)
  end
end