require 'helix_web_services_client/open_model'
require 'open-uri'

class HelixWebServicesClient
  # Create a new client workspace specification for a Helix Sync project.
  #
  # Returns a simple object with a 'client' property.
  def create_helix_sync_device_client(project_id, device, root)
    data = {
        device: device,
        root: root
    }
    obj = execute_method_with_body(:post, "/helix-sync/v1/#{URI.encode(project_id)}/clients/device", nil, data)
    OpenModel.new(obj)
  end

  def create_helix_sync_shelf_client(project_id)
    obj = execute_method_no_body(:post, "/helix-sync/v1/#{URI.encode(project_id)}/clients/shelf")
    OpenModel.new(obj)
  end

  def delete_helix_sync_device_client(project_id, device)
    obj = execute_method_no_body(:delete, "/helix-sync/v1/#{URI.encode(project_id)}/clients/device/#{URI.encode(device)}")
    OpenModel.new(obj)
  end

  def delete_helix_sync_shelf_client(project_id)
    obj = execute_method_no_body(:delete, "/helix-sync/v1/#{URI.encode(project_id)}/clients/shelf")
    OpenModel.new(obj)
  end

  # Retrieve the latest changelist.
  #
  # If the change doesn't exist, you'll get a ResourceNotFound exception.
  def fetch_helix_sync_latest_changelist(project_id)
    obj = execute_method_no_body(:get, "/helix-sync/v1/#{URI.encode(project_id)}/last-change")
    OpenModel.new(obj)
  end

  # Retrieve the pending changelist.
  #
  # If the change doesn't exist, you'll get a ResourceNotFound exception.
  def fetch_helix_sync_pending_changelist(project_id)
    obj = execute_method_no_body(:get, "/helix-sync/v1/#{URI.encode(project_id)}/pending")
    OpenModel.new(obj)
  end

  # Remove the pending changelist used for a Helix Sync project.
  def delete_helix_sync_pending_changelist(project_id)
    obj = execute_method_no_body(:delete, "/helix-sync/v1/#{URI.encode(project_id)}/pending")
    OpenModel.new(obj)
  end

  # "Preview", or generate the resolve plan, for the pending changelist.
  #
  # If the change doesn't exist, you'll get a ResourceNotFound exception.
  def preview_helix_sync_pending_change(project_id)
    arr = execute_method_no_body(:get, "/helix-sync/v1/#{URI.encode(project_id)}/preview")
    arr.map { |o| OpenModel.new(o) }
  end

  # Submits the pending changelist for the helix sync project.
  #
  # Currently this assumes that all files are shelved and nothing's open on
  # the client. This is likely not what we want ... but it's a start.
  #
  # @param project_id [String]
  def submit_helix_sync_pending_change(project_id)
    obj = execute_method_no_body(:post, "/helix-sync/v1/#{URI.encode(project_id)}/submit")
    OpenModel.new(obj)
  end
end
